// I could make an emum for a buy/sell/hold signal in a long-only strategy
// cool project from the android team at google:
//   https://google.github.io/comprehensive-rust/

enum IpAddrKind {
    V4,
    V6,
}

struct IpAddrOld {
    kind: IpAddrKind,
    address: String,
}

// A better way of representing IP in this case
enum IpAddrFunc {
    V4(String),
    V6(String),
}

// Even better way as these enums can hold
// different values, like a tuple, and like Haskell
enum IpAddr {
    V4(u8, u8, u8, u8),
    V6(String),
}

enum Message {
    Quit,
    Move { x: i32, y: i32 },
    Write(String),
    ChangeColor(i32, i32, i32),
}

impl Message {
    fn call(&self) {
        // method body here
    }
}

fn main() {
    let four = IpAddrKind::V4;
    let six = IpAddrKind::V6;

    route(four);
    route(six);

    let home_old: IpAddrOld = IpAddrOld {
        kind: IpAddrKind::V4,
        address: String::from("127.0.0.1"),
    };

    let loopback_old = IpAddrOld {
        kind: IpAddrKind::V6,
        address: String::from("::1"),
    };

    let home = IpAddrFunc::V4(String::from("127.0.0.1"));
    let loopback = IpAddrFunc::V6(String::from("::1"));

    let home_num = IpAddr::V4(127, 0, 0, 1);

    // Message section

    let m = Message::Write(String::from("hello"));
    m.call();

    // ------------------------------------------------------------------------------
    // Option enum section
    // Essentially Haskell's Maybe Monad
    // Rust even has a Prelude :pog:
    let some_number = Some(5);
    let some_char = Some('e');
    let absent_number: Option<i32> = None;
}

// takes either kind of IpAddrKind
fn route(ip_kind: IpAddrKind) {}
